import sys

from manageprojects.utilities import code_style
from manageprojects.utilities.subprocess_utils import verbose_check_call

from manage_django_project.config import project_info
from manage_django_project.management.base import BaseManageCommand


class Command(BaseManageCommand):
    help = 'Check/Fix project code style'

    def handle(self, *args, **options):
        verbose = options['verbosity'] > 1
        color = not options['no_color']

        project_root_path = project_info.config.project_root_path

        code_style._call_darker(package_root=project_root_path, color=color, verbose=verbose)
        verbose_check_call(
            'flake8',
            *args,
            cwd=project_root_path,
            exit_on_error=True,
        )
        print('Code style: OK')
        sys.exit(0)



from django.conf import settings
from django.test import TestCase


class ExampleProjectTestCase(TestCase):
    def test_settings(self):
        self.assertEqual(settings.SETTINGS_MODULE, 'manage_django_project_example.settings.tests')

    def test_index_page(self):
        response = self.client.get('/')
        self.assertRedirects(response, expected_url='/login/?next=/')



from rich.pretty import Pretty

from manage_django_project.config import project_info
from manage_django_project.management.base import BaseManageCommand


class Command(BaseManageCommand):
    help = 'Information about the current Django project'

    def handle(self, *args, **options):
        self.console.print('\nmanage_config = ', end='')
        self.console.print(Pretty(project_info))



from django_rich.management import RichCommand

from manage_django_project.config import project_info


class BaseManageCommand(RichCommand):
    help = ''

    def execute(self, *args, **options):
        if options['verbosity']:
            self.console.print('_' * self.console.width)
            self.console.print(f'[bold]{self.help}')

        project_info.assert_initialized()

        super().execute(*args, **options)


class BasePassManageCommand(RichCommand):
    help = ''

    def run_from_argv(self, argv):
        self.console.print('_' * self.console.width)
        self.console.print(f'[bold]{self.help}')



from manageprojects.utilities.subprocess_utils import verbose_check_call

from manage_django_project.config import project_info
from manage_django_project.management.base import BaseManageCommand


class Command(BaseManageCommand):
    help = 'Just install the project as editable via pip (Useful if version has been changed)'

    def handle(self, *args, **options):
        verbose_check_call('pip', 'install', '--no-deps', '-e', '.', cwd=project_info.config.project_root_path)



"""
    Just "Overwrite" the existing manage_django_project/management/commands/publish.py
    because we would like to publish "manage_django_project"
    and not "manage_django_project_example"
"""
from manageprojects.utilities.publish import publish_package

import manage_django_project
from manage_django_project.config import project_info
from manage_django_project.management.commands.publish import Command as OriginPublishCommand


class Command(OriginPublishCommand):
    def publish(self):
        publish_package(
            module=manage_django_project,  # <<< set the correct module!
            package_path=project_info.config.project_root_path,
        )



from django_rich.management import RichCommand
from manageprojects.utilities.subprocess_utils import verbose_check_call


class Command(RichCommand):
    help = 'Run safety check against current requirements files'

    def handle(self, *args, **options):
        self.console.print(f'\n[bold]{self.help}')

        verbose_check_call('safety', 'check', '-r', 'requirements.dev.txt')



# flake8: noqa: E405

"""
    Django settings for local development
"""

from .base import *  # noqa


# _____________________________________________________________________________
# Manage Django Project

INSTALLED_APPS.append('manage_django_project')
MANAGE_DJANGO_PROJECT_MODULE_NAME = 'manage_django_project_example'



from django.apps import AppConfig

from manage_django_project.config import project_info


class ManageDjangoProjectConfig(AppConfig):
    name = "manage_django_project"
    verbose_name = "Manage Django Project"

    def ready(self):
        project_info.assert_initialized()



"""
    Allow manage_django_project to be executable
    through `python -m manage_django_project`.
"""
from pathlib import Path

import manage_django_project_example
from manage_django_project.config import ManageConfig
from manage_django_project.manage import execute_django_from_command_line


def main():
    """
    entrypoint installed via pyproject.toml and [project.scripts] section.
    Must be set in ./manage.py and PROJECT_SHELL_SCRIPT
    """
    execute_django_from_command_line(
        config=ManageConfig(
            module=manage_django_project_example,
            project_root_path=Path(manage_django_project_example.__file__).parent.parent,
            prod_settings='manage_django_project_example.settings.prod',
            local_settings='manage_django_project_example.settings.local',
            test_settings='manage_django_project_example.settings.tests',
        )
    )


if __name__ == '__main__':
    main()



import os
import sys
from importlib import import_module

import cmd2
from cmd2 import Statement, categorize
from cmd2.constants import CMD_ATTR_ARGPARSER, CMD_ATTR_PRESERVE_QUOTES
from cmd2.decorators import _set_parser_prog
from django.core.management import BaseCommand, CommandParser, get_commands
from manageprojects.utilities.subprocess_utils import verbose_check_call
from rich import print
from rich.console import Console

from manage_django_project.config import project_info
from manage_django_project.management.base import BasePassManageCommand


class DjangoCommand:
    def __init__(self, *, command_name, package_name, console: Console):
        self.command_name = command_name
        self.package_name = package_name
        self.console = console

    def __call__(self, statement: Statement):
        """
        Call the Django command.
        Don't use django.core.management.call_command() because we can't switch the needed Django settings.
        """
        verbose = statement.args != '--help'

        if verbose:
            print(f'{statement=}')
            print('_' * self.console.width)
            print(f'[magenta]call command[/magenta] [cyan]{self.package_name}.[bold]{self.command_name}\n')

        # call our manage_django_project.manage.execute_django_from_command_line()
        # and pass all arguments
        args = [sys.executable, '-m', project_info.config.module.__name__, self.command_name]
        if arg_list := statement.arg_list:
            args.extend(arg_list)

        # Don't force any settings:
        env = {k: v for k, v in os.environ.items() if k != 'DJANGO_SETTINGS_MODULE'}

        verbose_check_call(*args, verbose=verbose, env=env)


class App(cmd2.Cmd):
    # Remove some default cmd2 commands:
    delattr(cmd2.Cmd, 'do_edit')
    delattr(cmd2.Cmd, 'do_shell')
    delattr(cmd2.Cmd, 'do_run_script')
    delattr(cmd2.Cmd, 'do_run_pyscript')

    def __init__(self, *args, console: Console, **kwargs):
        self.console = console

        for command_name, app_name in get_commands().items():
            if command_name == 'shell':
                continue

            cmd = DjangoCommand(command_name=command_name, package_name=app_name, console=self.console)

            categorize(func=cmd, category=app_name)

            pkg_name = f'{app_name}.management.commands.{command_name}'
            module = import_module(pkg_name)
            CommandClass = module.Command
            assert issubclass(CommandClass, BaseCommand)
            cmd.__doc__ = CommandClass.help

            cmd_instance: BaseCommand = CommandClass()
            parser: CommandParser = cmd_instance.create_parser(prog_name='manage.py', subcommand=command_name)

            _set_parser_prog(parser, prog='./manage.py')

            setattr(cmd, CMD_ATTR_ARGPARSER, parser)
            setattr(cmd, CMD_ATTR_PRESERVE_QUOTES, False)

            setattr(self, f'do_{command_name}', cmd)

        super().__init__(*args, stdout=self.console.file, **kwargs)

        self._startup_commands = ['help']

        # Set aliases:
        self.aliases.update({'q': 'quit'})

        # Display Tracebacks on errors:
        self.debug = True

        self.prompt = f'\n({project_info.distribution_name}) '


class Command(BasePassManageCommand):
    help = 'Go into cmd2 shell with all registered Django manage commands'

    def run_from_argv(self, argv):
        super().run_from_argv(argv)

        app = App(console=self.console)
        app.cmdloop()



from manageprojects.utilities.publish import publish_package
from manageprojects.utilities.subprocess_utils import verbose_check_call
from rich.pretty import pprint

from manage_django_project.config import project_info
from manage_django_project.management.base import BaseManageCommand


class Command(BaseManageCommand):
    help = 'Build a new release and publish it to PyPi'

    def handle(self, *args, **options):
        # don't publish if tests fail:
        verbose_check_call(
            './manage.py',
            'test',
            extra_env=dict(  # Use the test settings:
                DJANGO_SETTINGS_MODULE=project_info.config.test_settings,
            ),
        )
        # test pass -> publish:
        self.publish()

    def publish(self):
        kwargs = dict(
            module=project_info.config.module,
            package_path=project_info.config.project_root_path,
            distribution_name=project_info.distribution_name,
        )
        print('Start publishing with:')
        pprint(kwargs)
        publish_package(**kwargs)



from unittest.mock import patch

from django.core.management import call_command
from django.test import SimpleTestCase

import manage_django_project_example
from manage_django_project.management.commands import publish
from manage_django_project.tests import PROJECT_ROOT


class ConfigTestCase(SimpleTestCase):
    def test_basic(self):
        with patch.object(publish, 'publish_package') as publish_package_mock, patch.object(
            publish, 'verbose_check_call'
        ) as verbose_check_call_mock:
            call_command(publish.Command())
        publish_package_mock.assert_called_once_with(
            module=manage_django_project_example,
            package_path=PROJECT_ROOT,
            distribution_name='manage_django_project',
        )
        verbose_check_call_mock.assert_called_once_with(
            './manage.py',
            'test',
            extra_env={'DJANGO_SETTINGS_MODULE': 'manage_django_project_example.settings.tests'},
        )



from types import ModuleType

from django.core.management import call_command
from django.test import SimpleTestCase
from manageprojects.test_utils.project_setup import check_editor_config
from packaging.version import Version

from manage_django_project.config import project_info
from manage_django_project.management.commands import code_style


class ProjectSettingsTestCase(SimpleTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        project_info.assert_initialized()

    def test_version(self):
        self.assertIsInstance(project_info.module_version, Version)
        self.assertIsInstance(project_info.config.module, ModuleType)

        self.assertEqual(project_info.module_version, Version(project_info.config.module.__version__))

        pyproject_toml = project_info.get_pyproject_toml()
        pyproject_version = pyproject_toml['project']['version']

        self.assertEqual(project_info.config.module.__version__, pyproject_version)

    def test_check_editor_config(self):
        check_editor_config(package_root=project_info.config.project_root_path)

    def test_code_style(self):
        # Just run our django manage command that call's darker and flake8

        try:
            call_command(code_style.Command())
        except SystemExit as err:
            if err.code != 0:
                self.fail('Code style errors, see above!')



from django.apps import AppConfig


class ExampleProjectAppConfig(AppConfig):
    name = 'manage_django_project_example'
    verbose_name = 'Example Project'

    def ready(self):
        import manage_django_project_example.checks  # noqa



from django.test import SimpleTestCase

import manage_django_project_example
from manage_django_project.config import ManageConfig, project_info


class ConfigTestCase(SimpleTestCase):
    def test_basic(self):
        project_info.assert_initialized()

        self.assertEqual(project_info.distribution_name, 'manage_django_project')
        self.assertIsInstance(project_info.config, ManageConfig)
        self.assertEqual(project_info.config.module, manage_django_project_example)



from manageprojects.utilities.subprocess_utils import verbose_check_call

from manage_django_project.config import project_info
from manage_django_project.management.base import BaseManageCommand


def coverage_combine_report(verbose):
    cwd = project_info.config.project_root_path
    verbose_check_call('coverage', 'combine', '--append', verbose=verbose, exit_on_error=True, cwd=cwd)
    verbose_check_call('coverage', 'report', verbose=verbose, exit_on_error=True, cwd=cwd)
    verbose_check_call('coverage', 'xml', verbose=verbose, exit_on_error=True, cwd=cwd)
    verbose_check_call('coverage', 'json', verbose=verbose, exit_on_error=True, cwd=cwd)


class Command(BaseManageCommand):
    help = 'Run tests with coverage and report'

    def add_arguments(self, parser):
        parser.add_argument(
            "--no-processing",
            action="store_true",
            dest="no_processing",
            help='Run only coverage tests, but do not combine result and report',
        )
        parser.add_argument(
            "--context",
            help='The context label to record for this coverage run.',
        )

    def handle(self, *args, **options):
        cwd = project_info.config.project_root_path

        no_processing = options['no_processing']

        verbose = options['verbosity'] > 0

        old_coverage_data = cwd / '.coverage'
        old_coverage_data.unlink(missing_ok=True)

        args = ['coverage', 'run']
        if context := options['context']:
            args.append('--context')
            args.append(context)

        try:
            verbose_check_call(*args, verbose=verbose, exit_on_error=True, cwd=cwd)
        except SystemExit as err:
            if err.code != 0:
                raise  # No report if tests fails

        if not no_processing:
            coverage_combine_report(verbose)



import os
import sys

from manageprojects.utilities.version_info import print_version
from rich import print

from manage_django_project.config import ManageConfig, project_info


def execute_django_from_command_line(*, config: ManageConfig, argv=None):
    if argv is None:
        argv = sys.argv

    if len(argv) == 1:
        # Go into cmd2 shell if no command are given:
        argv.append('shell')

    project_info.initialize(config=config)

    print_version(module=project_info.config.module, project_root=project_info.config.project_root_path)
    print()

    if 'DJANGO_SETTINGS_MODULE' not in os.environ:
        if DJANGO_SETTINGS_MODULE := project_info.get_current_settings(argv):
            print(f'Set {DJANGO_SETTINGS_MODULE=}', file=sys.stderr)
            os.environ['DJANGO_SETTINGS_MODULE'] = DJANGO_SETTINGS_MODULE
        else:
            print('(Do not set DJANGO_SETTINGS_MODULE)')
    else:
        print(f'DJANGO_SETTINGS_MODULE={os.environ["DJANGO_SETTINGS_MODULE"]}', file=sys.stderr)

    print()

    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            'Couldn\'t import Django. Are you sure it\'s installed and '
            'available on your PYTHONPATH environment variable? Did you '
            'forget to activate a virtual environment?'
        ) from exc
    try:
        execute_from_command_line(argv)
    except Exception as err:
        from bx_py_utils.error_handling import print_exc_plus

        print_exc_plus(err)
        raise



import sys
from pathlib import Path

from bx_py_utils.path import assert_is_file
from django_rich.management import RichCommand
from manageprojects.utilities.subprocess_utils import verbose_check_call


class Command(RichCommand):
    help = 'Update project requirements via pip-tools'

    def handle(self, *args, **options):
        self.console.print(f'\n[bold]{self.help}')

        cwd = Path.cwd()
        assert_is_file(cwd / 'pyproject.toml')
        assert_is_file(cwd / 'requirements.txt')
        assert_is_file(cwd / 'requirements.dev.txt')

        bin_path = Path(sys.executable).parent

        verbose_check_call(bin_path / 'pip', 'install', '-U', 'pip')
        verbose_check_call(bin_path / 'pip', 'install', '-U', 'pip-tools')

        extra_env = dict(
            CUSTOM_COMPILE_COMMAND='./manage.py update_req',
        )

        pip_compile_base = [
            bin_path / 'pip-compile',
            '--verbose',
            '--allow-unsafe',  # https://pip-tools.readthedocs.io/en/latest/#deprecations
            '--resolver=backtracking',  # https://pip-tools.readthedocs.io/en/latest/#deprecations
            '--upgrade',
            '--generate-hashes',
        ]

        # Only "prod" dependencies:
        verbose_check_call(
            *pip_compile_base,
            'pyproject.toml',
            '--output-file',
            'requirements.txt',
            extra_env=extra_env,
        )

        # dependencies + "dev"-optional-dependencies:
        verbose_check_call(
            *pip_compile_base,
            'pyproject.toml',
            '--extra=dev',
            '--output-file',
            'requirements.dev.txt',
            extra_env=extra_env,
        )

        verbose_check_call('safety', 'check', '-r', 'requirements.dev.txt')

        # Install new dependencies in current .venv:
        verbose_check_call('pip-sync', 'requirements.dev.txt')



import sys

from manageprojects.utilities.subprocess_utils import verbose_check_call

from manage_django_project.management.base import BasePassManageCommand
from manage_django_project.management.commands.coverage import coverage_combine_report


class Command(BasePassManageCommand):
    help = 'Run tests via tox'

    def run_from_argv(self, argv):
        super().run_from_argv(argv)

        # Just pass every argument to the origin tox CLI:
        verbose_check_call(sys.executable, '-m', 'tox', *argv[2:])

        coverage_combine_report(verbose=True)

        sys.exit(0)



import dataclasses
from pathlib import Path
from types import ModuleType
from typing import Optional

from bx_py_utils.path import assert_is_file
from packaging.version import Version

from manage_django_project.management import commands


try:
    import tomllib  # New in Python 3.11
except ImportError:
    import tomli as tomllib


def get_own_commands():
    commands_path = Path(commands.__file__).parent
    own_commands = [item.stem for item in commands_path.glob('*.py') if not item.stem.startswith('_')]
    return own_commands


@dataclasses.dataclass
class ManageConfig:
    module: ModuleType
    project_root_path: Path

    # dotted name of Django settings for...
    prod_settings: str  # ... production
    local_settings: str  # ... run local Django dev. server
    test_settings: str  # ... run tests


@dataclasses.dataclass
class ProjectInfo:
    initialized: bool = False

    config: ManageConfig = None
    pyproject_toml_path: Optional[Path] = None
    distribution_name: Optional[str] = None
    module_version: Optional[Version] = None

    def initialize(self, config: ManageConfig):
        self.config = config

        self.module_version = Version(self.config.module.__version__)
        self.pyproject_toml_path = Path(self.config.project_root_path, 'pyproject.toml')

        pyproject_toml = self.get_pyproject_toml()
        self.distribution_name = pyproject_toml['project']['name']

        self.initialized = True

    def assert_initialized(self):
        assert self.initialized is True, f'No initialized: {self}'

    def get_pyproject_toml(self) -> dict:
        assert_is_file(self.pyproject_toml_path)
        pyproject_toml = tomllib.loads(self.pyproject_toml_path.read_text(encoding='UTF-8'))
        return pyproject_toml

    def get_settings_by_command(self, *, command_name) -> str:
        own_commands = get_own_commands()

        if command_name in ('test', 'coverage', 'tox'):
            return self.config.test_settings

        if command_name in own_commands:
            return self.config.local_settings

        return self.config.prod_settings

    def get_current_settings(self, argv) -> Optional[str]:
        for arg in argv:
            if arg.startswith('--settings'):
                # e.g.: Start a manage command with --settings option -> don't force any settings
                return

        settings_name = self.config.prod_settings

        if len(argv) == 1:
            # e.g.: just start "./manage.py"
            settings_name = self.config.local_settings
        elif len(argv) > 1:
            settings_name = self.get_settings_by_command(command_name=argv[1])

        return settings_name


project_info = ProjectInfo()



from django.conf import settings
from django.core.checks import Error, register


@register()
def check_settings(app_configs, **kwargs):
    errors = []
    if not settings.SETTINGS_MODULE.startswith('manage_django_project_example.settings.'):
        errors.append(
            Error(
                f'Wrong settings: {settings.SETTINGS_MODULE=} is not a manage_django_project_example settings',
                id='manage_django_project_example.E001',
            )
        )
    return errors
